/*
 * Copyright (C) 2022 NHR@FAU, University Erlangen-Nuremberg.
 * All rights reserved.
 * Use of this source code is governed by a MIT-style
 * license that can be found in the LICENSE file.
 */
#include <float.h>
#include <limits.h>
#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "parameter.h"
#include "solver.h"
#include "timing.h"

enum VARIANT { SOR = 1, RB, RBA };

int main(int argc, char** argv)
{
    int rank;
    int variant = RB;

    Parameter params;
    Solver solver;

    double S, E;

    MPI_Init(&argc, &argv);
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);
    initParameter(&params);

    if (argc < 2) {
        printf("Usage: %s <configFile>\n", argv[0]);
        exit(EXIT_SUCCESS);
    }

    readParameter(&params, argv[1]);
    if (argc == 3) {
        variant = atoi(argv[2]);
    }

    if (rank == 0) {
        printParameter(&params);
    }

    initSolver(&solver, &params, 2);

    S = getTimeStamp();

    switch (variant) {
    case SOR:
        printf("Plain SOR\n");
        solve(&solver);
        break;
    case RB:
        printf("Red-black SOR\n");
        solveRB(&solver);
        break;
    case RBA:
        printf("Red-black SOR with acceleration\n");
        solveRBA(&solver);
        break;
    }
    // getResult(&solver);

    E = getTimeStamp();

    if (rank == 0) {
        printf("Solution took %.2fs\n", E - S);
    }

    MPI_Finalize();
    return EXIT_SUCCESS;
}
